@@ -1,38 +1,31 @@
-/* echo.c, taken from Bash.
-   Copyright (C) 87, 89, 91, 92, 93, 1994 Free Software Foundation, Inc.
+/* echo.c, derived from code echo.c in Bash.
+   Copyright (C) 87,89, 1991-1997, 1999 Free Software Foundation, Inc.
 
-This file is part of GNU Bash, the Bourne Again SHell.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-Bash is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
-version.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-Bash is distributed in the hope that it will be useful, but WITHOUT ANY
-WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-You should have received a copy of the GNU General Public License along
-with Bash; see the file COPYING.  If not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <sys/types.h>
 #include "system.h"
 #include "long-options.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "echo"
+
+#define AUTHORS "FIXME unknown"
+
 /* echo [-neE] [arg ...]
 Output the ARGs.  If -n is specified, the trailing newline is
 suppressed.  If the -e option is given, interpretation of the
@@ -57,36 +50,37 @@ on System V systems with the -E option.
 
 /* If defined, interpret backslash escapes unless -E is given.
    V9_ECHO must also be defined.  */
-#define V9_DEFAULT
+/* #define V9_DEFAULT */
 
 #if defined (V9_ECHO)
-#  if defined (V9_DEFAULT)
-#    define VALID_ECHO_OPTIONS "neE"
-#  else
-#    define VALID_ECHO_OPTIONS "ne"
-#  endif /* !V9_DEFAULT */
+# if defined (V9_DEFAULT)
+#  define VALID_ECHO_OPTIONS "neE"
+# else
+#  define VALID_ECHO_OPTIONS "ne"
+# endif /* !V9_DEFAULT */
 #else /* !V9_ECHO */
-#  define VALID_ECHO_OPTIONS "n"
+# define VALID_ECHO_OPTIONS "n"
 #endif /* !V9_ECHO */
 
 /* The name this program was run with. */
 char *program_name;
 
-static void
-usage (status)
-     int status;
+void
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("Usage: %s [OPTION]... [STRING]...\n", program_name);
-      printf ("\
+      printf (_("Usage: %s [OPTION]... [STRING]...\n"), program_name);
+      printf (_("\
+Echo the STRING(s) to standard output.\n\
 \n\
   -n              do not output the trailing newline\n\
-  -e              (unused)\n\
-  -E              disable interpolation of some sequences in STRINGs\n\
+  -e              enable interpretation of the backslash-escaped characters\n\
+                    listed below\n\
+  -E              disable interpretation of those sequences in STRINGs\n\
       --help      display this help and exit (should be alone)\n\
       --version   output version information and exit (should be alone)\n\
 \n\
@@ -102,7 +96,8 @@ Without -E, the following sequences are recognized and interpolated:\n\
   \\r     carriage return\n\
   \\t     horizontal tab\n\
   \\v     vertical tab\n\
-");
+"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
@@ -110,22 +105,30 @@ Without -E, the following sequences are recognized and interpolated:\n\
 /* Print the words in LIST to standard output.  If the first word is
    `-n', then don't print a trailing newline.  We also support the
    echo syntax from Version 9 unix systems. */
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+
+int
+main (int argc, char **argv)
 {
   int display_return = 1, do_v9 = 0;
+  int allow_options = 1;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, usage);
+  /* Don't recognize --help or --version if POSIXLY_CORRECT is set.  */
+  if (getenv ("POSIXLY_CORRECT") == NULL)
+    parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
+  else
+    allow_options = 0;
 
 /* System V machines already have a /bin/sh with a v9 behaviour.  We
    use the identical behaviour for these machines so that the
    existing system shell scripts won't barf. */
 #if defined (V9_ECHO) && defined (V9_DEFAULT)
-  do_v9 = 1;
+  do_v9 = allow_options;
 #endif
 
   --argc;
@@ -143,7 +146,7 @@ main (argc, argv)
 
       for (i = 0; temp[i]; i++)
 	{
-	  if (rindex (VALID_ECHO_OPTIONS, temp[i]) == 0)
+	  if (strrchr (VALID_ECHO_OPTIONS, temp[i]) == 0)
 	    goto just_echo;
 	}
 
@@ -154,15 +157,15 @@ main (argc, argv)
 	 Handle them. */
       while (*temp)
 	{
-	  if (*temp == 'n')
+	  if (allow_options && *temp == 'n')
 	    display_return = 0;
 #if defined (V9_ECHO)
-	  else if (*temp == 'e')
+	  else if (allow_options && *temp == 'e')
 	    do_v9 = 1;
-#if defined (V9_DEFAULT)
-	  else if (*temp == 'E')
+# if defined (V9_DEFAULT)
+	  else if (allow_options && *temp == 'E')
 	    do_v9 = 0;
-#endif /* V9_DEFAULT */
+# endif /* V9_DEFAULT */
 #endif /* V9_ECHO */
 	  else
 	    goto just_echo;
