@@ -1,5 +1,5 @@
 /* expr -- evaluate expressions.
-   Copyright (C) 1986, 1991 Free Software Foundation, Inc.
+   Copyright (C) 86, 1991-1997, 1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Author: Mike Parker.
 
@@ -28,33 +28,19 @@
 
    Define EVAL_TRACE to print an evaluation trace.  */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
-#include <regex.h>
-
 #include "system.h"
-#include "version.h"
+
+#include <regex.h>
 #include "long-options.h"
+#include "error.h"
 
-#if defined (STDC_HEADERS) || (!defined (isascii) && !defined (HAVE_ISASCII))
-#define ISASCII(c) 1
-#else
-#define ISASCII(c) isascii(c)
-#endif
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "expr"
 
-#define ISDIGIT(c) (ISASCII (c) && isdigit (c))
+#define AUTHORS "Mike Parker"
 
 #define NEW(type) ((type *) xmalloc (sizeof (type)))
 #define OLD(x) free ((char *) x)
@@ -79,56 +65,56 @@ struct valinfo
 };
 typedef struct valinfo VALUE;
 
+/* Non-zero if the POSIXLY_CORRECT environment variable is set.
+   The unary operator `quote' is disabled when this variable is zero.  */
+static int posixly_correct;
+
 /* The arguments given to the program, minus the program name.  */
 static char **args;
 
 /* The name this program was run with. */
 char *program_name;
 
-void error ();
 char *xstrdup ();
-char *strstr ();
-char *xmalloc ();
 
-static VALUE *docolon ();
-static VALUE *eval ();
-static VALUE *int_value ();
-static VALUE *str_value ();
-static int isstring ();
-static int nextarg ();
-static int nomoreargs ();
-static int null ();
-static int toarith ();
-static void freev ();
-static void printv ();
-static void tostring ();
+static VALUE *docolon PARAMS ((VALUE *sv, VALUE *pv));
+static VALUE *eval PARAMS ((void));
+static VALUE *int_value PARAMS ((int i));
+static VALUE *str_value PARAMS ((char *s));
+static int isstring PARAMS ((VALUE *v));
+static int nextarg PARAMS ((char *str));
+static int nomoreargs PARAMS ((void));
+static int null PARAMS ((VALUE *v));
+static int toarith PARAMS ((VALUE *v));
+static void freev PARAMS ((VALUE *v));
+static void printv PARAMS ((VALUE *v));
+static void tostring PARAMS ((VALUE *v));
 
 #ifdef EVAL_TRACE
 static void trace ();
 #endif
 
-static void
-usage (status)
-     int status;
+void
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s EXPRESSION\n\
   or:  %s OPTION\n\
-",
+"),
 	      program_name, program_name);
-      printf ("\
+      printf (_("\
 \n\
   --help      display this help and exit\n\
   --version   output version information and exit\n\
 \n\
-");
-      printf ("\
-EXPRESSION value is written on standard output.  A white line\n\
+"));
+      printf (_("\
+Print the value of EXPRESSION to standard output.  A blank line below\n\
 separates increasing precedence groups.  EXPRESSION may be:\n\
 \n\
   ARG1 | ARG2       ARG1 if it is neither null nor 0, otherwise ARG2\n\
@@ -155,38 +141,51 @@ separates increasing precedence groups.  EXPRESSION may be:\n\
   substr STRING POS LENGTH   substring of STRING, POS counted from 1\n\
   index STRING CHARS         index in STRING where any CHARS is found, or 0\n\
   length STRING              length of STRING\n\
+  quote TOKEN                interpret TOKEN as a string, even if it is a\n\
+                               keyword like `match' or an operator like `/'\n\
 \n\
   ( EXPRESSION )             value of EXPRESSION\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
-Beware that some operators need to be escaped by backslashes for shells.\n\
+Beware that many operators need to be escaped or quoted for shells.\n\
 Comparisons are arithmetic if both ARGs are numbers, else lexicographical.\n\
 Pattern matches return the string matched between \\( and \\) or null; if\n\
 \\( and \\) are not used, they return the number of characters matched or 0.\n\
-");
+"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
   VALUE *v;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, usage);
+  posixly_correct = (getenv ("POSIXLY_CORRECT") != NULL);
+
+  /* Recognize --help or --version only if POSIXLY_CORRECT is not set.  */
+  if (!posixly_correct)
+    parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+			AUTHORS, usage);
 
   if (argc == 1)
-    usage (1);
+    {
+      error (0, 0, _("too few arguments"));
+      usage (1);
+    }
+
   args = argv + 1;
 
   v = eval ();
   if (!nomoreargs ())
-    error (2, 0, "syntax error");
+    error (2, 0, _("syntax error"));
   printv (v);
 
   exit (null (v));
@@ -195,8 +194,7 @@ main (argc, argv)
 /* Return a VALUE for I.  */
 
 static VALUE *
-int_value (i)
-     int i;
+int_value (int i)
 {
   VALUE *v;
 
@@ -209,8 +207,7 @@ int_value (i)
 /* Return a VALUE for S.  */
 
 static VALUE *
-str_value (s)
-     char *s;
+str_value (char *s)
 {
   VALUE *v;
 
@@ -223,8 +220,7 @@ str_value (s)
 /* Free VALUE V, including structure components.  */
 
 static void
-freev (v)
-     VALUE *v;
+freev (VALUE *v)
 {
   if (v->type == string)
     free (v->u.s);
@@ -234,8 +230,7 @@ freev (v)
 /* Print VALUE V.  */
 
 static void
-printv (v)
-     VALUE *v;
+printv (VALUE *v)
 {
   switch (v->type)
     {
@@ -253,15 +248,14 @@ printv (v)
 /* Return nonzero if V is a null-string or zero-number.  */
 
 static int
-null (v)
-     VALUE *v;
+null (VALUE *v)
 {
   switch (v->type)
     {
     case integer:
       return v->u.i == 0;
     case string:
-      return v->u.s[0] == '\0' || strcmp(v->u.s, "0") == 0;
+      return v->u.s[0] == '\0' || strcmp (v->u.s, "0") == 0;
     default:
       abort ();
     }
@@ -270,8 +264,7 @@ null (v)
 /* Return nonzero if V is a string value.  */
 
 static int
-isstring (v)
-     VALUE *v;
+isstring (VALUE *v)
 {
   return v->type == string;
 }
@@ -279,8 +272,7 @@ isstring (v)
 /* Coerce V to a string value (can't fail).  */
 
 static void
-tostring (v)
-     VALUE *v;
+tostring (VALUE *v)
 {
   char *temp;
 
@@ -302,8 +294,7 @@ tostring (v)
 /* Coerce V to an integer value.  Return 1 on success, 0 on failure.  */
 
 static int
-toarith (v)
-     VALUE *v;
+toarith (VALUE *v)
 {
   int i;
   int neg;
@@ -342,8 +333,7 @@ toarith (v)
    STR must not be NULL.  */
 
 static int
-nextarg (str)
-     char *str;
+nextarg (char *str)
 {
   if (*args == NULL)
     return 0;
@@ -353,7 +343,7 @@ nextarg (str)
 /* Return nonzero if there no more tokens.  */
 
 static int
-nomoreargs ()
+nomoreargs (void)
 {
   return *args == 0;
 }
@@ -373,12 +363,12 @@ int name (l, r) VALUE *l; VALUE *r;		\
  else						\
    return l->u.i rel r->u.i;			\
 }
-cmpf (less_than, <)
-cmpf (less_equal, <=)
-cmpf (equal, ==)
-cmpf (not_equal, !=)
-cmpf (greater_equal, >=)
-cmpf (greater_than, >)
+ cmpf (less_than, <)
+ cmpf (less_equal, <=)
+ cmpf (equal, ==)
+ cmpf (not_equal, !=)
+ cmpf (greater_equal, >=)
+ cmpf (greater_than, >)
 
 #undef cmpf
 
@@ -389,7 +379,7 @@ static						\
 int name (l, r) VALUE *l; VALUE *r;		\
 {						\
   if (!toarith (l) || !toarith (r))		\
-    error (2, 0, "non-numeric argument");	\
+    error (2, 0, _("non-numeric argument"));	\
   return l->u.i op r->u.i;			\
 }
 
@@ -397,17 +387,17 @@ int name (l, r) VALUE *l; VALUE *r;		\
 int name (l, r) VALUE *l; VALUE *r;		\
 {						\
   if (!toarith (l) || !toarith (r))		\
-    error (2, 0, "non-numeric argument");	\
+    error (2, 0, _("non-numeric argument"));	\
   if (r->u.i == 0)				\
-    error (2, 0, "division by zero");		\
+    error (2, 0, _("division by zero"));		\
   return l->u.i op r->u.i;			\
 }
 
-arithf (plus, +)
-arithf (minus, -)
-arithf (multiply, *)
-arithdivf (divide, /)
-arithdivf (mod, %)
+ arithf (plus, +)
+ arithf (minus, -)
+ arithf (multiply, *)
+ arithdivf (divide, /)
+ arithdivf (mod, %)
 
 #undef arithf
 #undef arithdivf
@@ -433,9 +423,7 @@ trace (fxn)
    PV is the VALUE for the rhs (the pattern).  */
 
 static VALUE *
-docolon (sv, pv)
-     VALUE *sv;
-     VALUE *pv;
+docolon (VALUE *sv, VALUE *pv)
 {
   VALUE *v;
   const char *errmsg;
@@ -446,10 +434,21 @@ docolon (sv, pv)
   tostring (sv);
   tostring (pv);
 
+  if (pv->u.s[0] == '^')
+    {
+      error (0, 0, _("\
+warning: unportable BRE: `%s': using `^' as the first character\n\
+of the basic regular expression is not portable; it is being ignored"),
+	     pv->u.s);
+    }
+
   len = strlen (pv->u.s);
+  memset (&re_buffer, 0, sizeof (re_buffer));
+  memset (&re_regs, 0, sizeof (re_regs));
   re_buffer.allocated = 2 * len;
   re_buffer.buffer = (unsigned char *) xmalloc (re_buffer.allocated);
   re_buffer.translate = 0;
+  re_syntax_options = RE_SYNTAX_POSIX_BASIC;
   errmsg = re_compile_pattern (pv->u.s, len, &re_buffer);
   if (errmsg)
     error (2, 0, "%s", errmsg);
@@ -469,7 +468,7 @@ docolon (sv, pv)
   else
     {
       /* Match failed -- return the right kind of null.  */
-      if (strstr (pv->u.s, "\\("))
+      if (re_buffer.re_nsub > 0)
 	v = str_value ("");
       else
 	v = int_value (0);
@@ -481,7 +480,7 @@ docolon (sv, pv)
 /* Handle bare operands and ( expr ) syntax.  */
 
 static VALUE *
-eval7 ()
+eval7 (void)
 {
   VALUE *v;
 
@@ -489,28 +488,28 @@ eval7 ()
   trace ("eval7");
 #endif
   if (nomoreargs ())
-    error (2, 0, "syntax error");
+    error (2, 0, _("syntax error"));
 
   if (nextarg ("("))
     {
       args++;
       v = eval ();
       if (!nextarg (")"))
-	error (2, 0, "syntax error");
+	error (2, 0, _("syntax error"));
       args++;
       return v;
     }
 
   if (nextarg (")"))
-    error (2, 0, "syntax error");
+    error (2, 0, _("syntax error"));
 
   return str_value (*args++);
 }
 
-/* Handle match, substr, index, and length keywords.  */
+/* Handle match, substr, index, length, and quote keywords.  */
 
 static VALUE *
-eval6 ()
+eval6 (void)
 {
   VALUE *l;
   VALUE *r;
@@ -521,7 +520,14 @@ eval6 ()
 #ifdef EVAL_TRACE
   trace ("eval6");
 #endif
-  if (nextarg ("length"))
+  if (!posixly_correct && nextarg ("quote"))
+    {
+      args++;
+      if (nomoreargs ())
+	error (2, 0, _("syntax error"));
+      return str_value (*args++);
+    }
+  else if (nextarg ("length"))
     {
       args++;
       r = eval6 ();
@@ -548,7 +554,7 @@ eval6 ()
       tostring (l);
       tostring (r);
       v = int_value (strcspn (l->u.s, r->u.s) + 1);
-      if (v->u.i == strlen (l->u.s) + 1)
+      if (v->u.i == (int) strlen (l->u.s) + 1)
 	v->u.i = 0;
       freev (l);
       freev (r);
@@ -562,7 +568,7 @@ eval6 ()
       i2 = eval6 ();
       tostring (l);
       if (!toarith (i1) || !toarith (i2)
-	  || i1->u.i > strlen (l->u.s)
+	  || i1->u.i > (int) strlen (l->u.s)
 	  || i1->u.i <= 0 || i2->u.i <= 0)
 	v = str_value ("");
       else
@@ -586,7 +592,7 @@ eval6 ()
    Calls docolon to do the real work.  */
 
 static VALUE *
-eval5 ()
+eval5 (void)
 {
   VALUE *l;
   VALUE *r;
@@ -615,7 +621,7 @@ eval5 ()
 /* Handle *, /, % operators.  */
 
 static VALUE *
-eval4 ()
+eval4 (void)
 {
   VALUE *l;
   VALUE *r;
@@ -648,7 +654,7 @@ eval4 ()
 /* Handle +, - operators.  */
 
 static VALUE *
-eval3 ()
+eval3 (void)
 {
   VALUE *l;
   VALUE *r;
@@ -679,7 +685,7 @@ eval3 ()
 /* Handle comparisons.  */
 
 static VALUE *
-eval2 ()
+eval2 (void)
 {
   VALUE *l;
   VALUE *r;
@@ -720,7 +726,7 @@ eval2 ()
 /* Handle &.  */
 
 static VALUE *
-eval1 ()
+eval1 (void)
 {
   VALUE *l;
   VALUE *r;
@@ -752,7 +758,7 @@ eval1 ()
 /* Handle |.  */
 
 static VALUE *
-eval ()
+eval (void)
 {
   VALUE *l;
   VALUE *r;
