@@ -1,5 +1,5 @@
 /* printf - format and print data
-   Copyright (C) 90, 91, 92, 93, 1994 Free Software Foundation, Inc.
+   Copyright (C) 1990-1998, Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Usage: printf format [argument...]
 
@@ -43,79 +43,61 @@
 
    David MacKenzie <djm@gnu.ai.mit.edu> */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <getopt.h>
 
 #include "system.h"
-#include "version.h"
 #include "long-options.h"
+#include "error.h"
 
-#if defined (STDC_HEADERS) || (!defined (isascii) && !defined (HAVE_ISASCII))
-#define ISASCII(c) 1
-#else
-#define ISASCII(c) isascii(c)
-#endif
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "printf"
 
-#define ISDIGIT(c) (ISASCII (c) && isdigit (c))
-#define ISXDIGIT(c) (ISASCII (c) && isxdigit (c))
+#define AUTHORS "David MacKenzie"
 
 #ifndef STDC_HEADERS
 double strtod ();
-long strtol ();
-unsigned long strtoul ();
+long int strtol ();
+unsigned long int strtoul ();
 #endif
 
 #define isodigit(c) ((c) >= '0' && (c) <= '7')
-#define hextobin(c) ((c)>='a'&&(c)<='f' ? (c)-'a'+10 : (c)>='A'&&(c)<='F' ? (c)-'A'+10 : (c)-'0')
+#define hextobin(c) ((c) >= 'a' && (c) <= 'f' ? (c) - 'a' + 10 : \
+		     (c) >= 'A' && (c) <= 'F' ? (c) - 'A' + 10 : (c) - '0')
 #define octtobin(c) ((c) - '0')
 
-char *xmalloc ();
-void error ();
-
-static double xstrtod ();
-static int print_esc ();
-static int print_formatted ();
-static long xstrtol ();
-static unsigned long xstrtoul ();
-static void print_direc ();
-static void print_esc_char ();
-static void print_esc_string ();
-static void verify ();
-
 /* The value to return to the calling program.  */
 static int exit_status;
 
+/* Non-zero if the POSIXLY_CORRECT environment variable is set.  */
+static int posixly_correct;
+
+/* This message appears in N_() here rather than just in _() below because
+   the sole use would have been in a #define, and xgettext doesn't look for
+   strings in cpp directives.  */
+static char *const cfcc_msg =
+ N_("warning: %s: character(s) following character constant have been ignored");
+
 /* The name this program was run with. */
 char *program_name;
 
-static void
-usage (status)
-     int status;
+void
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s FORMAT [ARGUMENT]...\n\
   or:  %s OPTION\n\
-",
+"),
 	      program_name, program_name);
-      printf ("\
+      printf (_("\
+Print ARGUMENT(s) according to FORMAT.\n\
 \n\
   --help      display this help and exit\n\
   --version   output version information and exit\n\
@@ -140,208 +122,65 @@ FORMAT controls the output as in C printf.  Interpreted sequences are:\n\
 \n\
 and all C format specifications ending with one of diouxXfeEgGcs, with\n\
 ARGUMENTs converted to proper type first.  Variable widths are handled.\n\
-");
+"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+static void
+verify (const char *s, const char *end)
 {
-  char *format;
-  int args_used;
-
-  program_name = argv[0];
-  exit_status = 0;
-
-  parse_long_options (argc, argv, usage);
-
-  if (argc == 1)
+  if (errno)
     {
-      fprintf (stderr, "Usage: %s format [argument...]\n", program_name);
-      exit (1);
+      error (0, errno, "%s", s);
+      exit_status = 1;
     }
-
-  format = argv[1];
-  argc -= 2;
-  argv += 2;
-
-  do
+  else if (*end)
     {
-      args_used = print_formatted (format, argc, argv);
-      argc -= args_used;
-      argv += args_used;
+      if (s == end)
+	error (0, 0, _("%s: expected a numeric value"), s);
+      else
+	error (0, 0, _("%s: value not completely converted"), s);
+      exit_status = 1;
     }
-  while (args_used > 0 && argc > 0);
-
-  exit (exit_status);
 }
 
-/* Print the text in FORMAT, using ARGV (with ARGC elements) for
-   arguments to any `%' directives.
-   Return the number of elements of ARGV used.  */
+#define STRTOX(TYPE, FUNC_NAME, LIB_FUNC_EXPR)				 \
+static TYPE								 \
+FUNC_NAME (s)								 \
+     const char *s;							 \
+{									 \
+  char *end;								 \
+  TYPE val;								 \
+									 \
+  if (*s == '\"' || *s == '\'')						 \
+    {									 \
+      val = *(unsigned char *) ++s;					 \
+      /* If POSIXLY_CORRECT is not set, then give a warning that there	 \
+	 are characters following the character constant and that GNU	 \
+	 printf is ignoring those characters.  If POSIXLY_CORRECT *is*	 \
+	 set, then don't give the warning.  */				 \
+      if (*++s != 0 && !posixly_correct)				 \
+	error (0, 0, _(cfcc_msg), s);					 \
+    }									 \
+  else									 \
+    {									 \
+      errno = 0;							 \
+      val = LIB_FUNC_EXPR;						 \
+      verify (s, end);							 \
+    }									 \
+  return val;								 \
+}									 \
 
-static int
-print_formatted (format, argc, argv)
-     char *format;
-     int argc;
-     char **argv;
-{
-  int save_argc = argc;		/* Preserve original value.  */
-  char *f;			/* Pointer into `format'.  */
-  char *direc_start;		/* Start of % directive.  */
-  int direc_length;		/* Length of % directive.  */
-  int field_width;		/* Arg to first '*', or -1 if none.  */
-  int precision;		/* Arg to second '*', or -1 if none.  */
-
-  for (f = format; *f; ++f)
-    {
-      switch (*f)
-	{
-	case '%':
-	  direc_start = f++;
-	  direc_length = 1;
-	  field_width = precision = -1;
-	  if (*f == '%')
-	    {
-	      putchar ('%');
-	      break;
-	    }
-	  if (*f == 'b')
-	    {
-	      if (argc > 0)
-		{
-		  print_esc_string (*argv);
-		  ++argv;
-		  --argc;
-		}
-	      break;
-	    }
-	  if (index ("-+ #", *f))
-	    {
-	      ++f;
-	      ++direc_length;
-	    }
-	  if (*f == '*')
-	    {
-	      ++f;
-	      ++direc_length;
-	      if (argc > 0)
-		{
-		  field_width = xstrtoul (*argv);
-		  ++argv;
-		  --argc;
-		}
-	      else
-		field_width = 0;
-	    }
-	  else
-	    while (ISDIGIT (*f))
-	      {
-		++f;
-		++direc_length;
-	      }
-	  if (*f == '.')
-	    {
-	      ++f;
-	      ++direc_length;
-	      if (*f == '*')
-		{
-		  ++f;
-		  ++direc_length;
-		  if (argc > 0)
-		    {
-		      precision = xstrtoul (*argv);
-		      ++argv;
-		      --argc;
-		    }
-		  else
-		    precision = 0;
-		}
-	      else
-		while (ISDIGIT (*f))
-		  {
-		    ++f;
-		    ++direc_length;
-		  }
-	    }
-	  if (*f == 'l' || *f == 'L' || *f == 'h')
-	    {
-	      ++f;
-	      ++direc_length;
-	    }
-	  if (!index ("diouxXfeEgGcs", *f))
-	    error (1, 0, "%%%c: invalid directive", *f);
-	  ++direc_length;
-	  if (argc > 0)
-	    {
-	      print_direc (direc_start, direc_length, field_width,
-			   precision, *argv);
-	      ++argv;
-	      --argc;
-	    }
-	  else
-	    print_direc (direc_start, direc_length, field_width,
-			 precision, "");
-	  break;
-
-	case '\\':
-	  f += print_esc (f);
-	  break;
-
-	default:
-	  putchar (*f);
-	}
-    }
-
-  return save_argc - argc;
-}
-
-/* Print a \ escape sequence starting at ESCSTART.
-   Return the number of characters in the escape sequence
-   besides the backslash. */
-
-static int
-print_esc (escstart)
-     char *escstart;
-{
-  register char *p = escstart + 1;
-  int esc_value = 0;		/* Value of \nnn escape. */
-  int esc_length;		/* Length of \nnn escape. */
-
-  /* \0ooo and \xhhh escapes have maximum length of 3 chars. */
-  if (*p == 'x')
-    {
-      for (esc_length = 0, ++p;
-	   esc_length < 3 && ISXDIGIT (*p);
-	   ++esc_length, ++p)
-	esc_value = esc_value * 16 + hextobin (*p);
-      if (esc_length == 0)
-	error (1, 0, "missing hexadecimal number in escape");
-      putchar (esc_value);
-    }
-  else if (*p == '0')
-    {
-      for (esc_length = 0, ++p;
-	   esc_length < 3 && isodigit (*p);
-	   ++esc_length, ++p)
-	esc_value = esc_value * 8 + octtobin (*p);
-      putchar (esc_value);
-    }
-  else if (index ("\"\\abcfnrtv", *p))
-    print_esc_char (*p++);
-  else
-    error (1, 0, "\\%c: invalid escape", *p);
-  return p - escstart - 1;
-}
+STRTOX (unsigned long int, xstrtoul, (strtoul (s, &end, 0)))
+STRTOX (long int,          xstrtol,  (strtol  (s, &end, 0)))
+STRTOX (double,            xstrtod,  (strtod  (s, &end)))
 
 /* Output a single-character \ escape.  */
 
 static void
-print_esc_char (c)
-     char c;
+print_esc_char (int c)
 {
   switch (c)
     {
@@ -375,11 +214,47 @@ print_esc_char (c)
     }
 }
 
+/* Print a \ escape sequence starting at ESCSTART.
+   Return the number of characters in the escape sequence
+   besides the backslash. */
+
+static int
+print_esc (const char *escstart)
+{
+  register const char *p = escstart + 1;
+  int esc_value = 0;		/* Value of \nnn escape. */
+  int esc_length;		/* Length of \nnn escape. */
+
+  /* \0ooo and \xhhh escapes have maximum length of 3 chars. */
+  if (*p == 'x')
+    {
+      for (esc_length = 0, ++p;
+	   esc_length < 3 && ISXDIGIT (*p);
+	   ++esc_length, ++p)
+	esc_value = esc_value * 16 + hextobin (*p);
+      if (esc_length == 0)
+	error (1, 0, _("missing hexadecimal number in escape"));
+      putchar (esc_value);
+    }
+  else if (*p == '0')
+    {
+      for (esc_length = 0, ++p;
+	   esc_length < 3 && isodigit (*p);
+	   ++esc_length, ++p)
+	esc_value = esc_value * 8 + octtobin (*p);
+      putchar (esc_value);
+    }
+  else if (strchr ("\"\\abcfnrtv", *p))
+    print_esc_char (*p++);
+  else
+    error (1, 0, _("\\%c: invalid escape"), *p);
+  return p - escstart - 1;
+}
+
 /* Print string STR, evaluating \ escapes. */
 
 static void
-print_esc_string (str)
-     char *str;
+print_esc_string (const char *str)
 {
   for (; *str; str++)
     if (*str == '\\')
@@ -394,12 +269,8 @@ print_esc_string (str)
    '*' values in those fields. */
 
 static void
-print_direc (start, length, field_width, precision, argument)
-     char *start;
-     int length;
-     int field_width;
-     int precision;
-     char *argument;
+print_direc (const char *start, size_t length, int field_width,
+	     int precision, const char *argument)
 {
   char *p;		/* Null-terminated copy of % directive. */
 
@@ -493,60 +364,162 @@ print_direc (start, length, field_width, precision, argument)
   free (p);
 }
 
-static unsigned long
-xstrtoul (s)
-     char *s;
+/* Print the text in FORMAT, using ARGV (with ARGC elements) for
+   arguments to any `%' directives.
+   Return the number of elements of ARGV used.  */
+
+static int
+print_formatted (const char *format, int argc, char **argv)
 {
-  char *end;
-  unsigned long val;
+  int save_argc = argc;		/* Preserve original value.  */
+  const char *f;		/* Pointer into `format'.  */
+  const char *direc_start;	/* Start of % directive.  */
+  size_t direc_length;		/* Length of % directive.  */
+  int field_width;		/* Arg to first '*', or -1 if none.  */
+  int precision;		/* Arg to second '*', or -1 if none.  */
 
-  errno = 0;
-  val = strtoul (s, &end, 0);
-  verify (s, end);
-  return val;
-}
-
-static long
-xstrtol (s)
-     char *s;
-{
-  char *end;
-  long val;
-
-  errno = 0;
-  val = strtol (s, &end, 0);
-  verify (s, end);
-  return val;
-}
-
-static double
-xstrtod (s)
-     char *s;
-{
-  char *end;
-  double val;
-
-  errno = 0;
-  val = strtod (s, &end);
-  verify (s, end);
-  return val;
-}
-
-static void
-verify (s, end)
-     char *s, *end;
-{
-  if (errno)
+  for (f = format; *f; ++f)
     {
-      error (0, errno, "%s", s);
-      exit_status = 1;
-    }
-  else if (*end)
-    {
-      if (s == end)
-	error (0, 0, "%s: expected a numeric value", s);
-      else
-	error (0, 0, "%s: value not completely converted", s);
-      exit_status = 1;
+      switch (*f)
+	{
+	case '%':
+	  direc_start = f++;
+	  direc_length = 1;
+	  field_width = precision = -1;
+	  if (*f == '%')
+	    {
+	      putchar ('%');
+	      break;
+	    }
+	  if (*f == 'b')
+	    {
+	      if (argc > 0)
+		{
+		  print_esc_string (*argv);
+		  ++argv;
+		  --argc;
+		}
+	      break;
+	    }
+	  if (strchr ("-+ #", *f))
+	    {
+	      ++f;
+	      ++direc_length;
+	    }
+	  if (*f == '*')
+	    {
+	      ++f;
+	      ++direc_length;
+	      if (argc > 0)
+		{
+		  field_width = xstrtoul (*argv);
+		  ++argv;
+		  --argc;
+		}
+	      else
+		field_width = 0;
+	    }
+	  else
+	    while (ISDIGIT (*f))
+	      {
+		++f;
+		++direc_length;
+	      }
+	  if (*f == '.')
+	    {
+	      ++f;
+	      ++direc_length;
+	      if (*f == '*')
+		{
+		  ++f;
+		  ++direc_length;
+		  if (argc > 0)
+		    {
+		      precision = xstrtoul (*argv);
+		      ++argv;
+		      --argc;
+		    }
+		  else
+		    precision = 0;
+		}
+	      else
+		while (ISDIGIT (*f))
+		  {
+		    ++f;
+		    ++direc_length;
+		  }
+	    }
+	  if (*f == 'l' || *f == 'L' || *f == 'h')
+	    {
+	      ++f;
+	      ++direc_length;
+	    }
+	  if (!strchr ("diouxXfeEgGcs", *f))
+	    error (1, 0, _("%%%c: invalid directive"), *f);
+	  ++direc_length;
+	  if (argc > 0)
+	    {
+	      print_direc (direc_start, direc_length, field_width,
+			   precision, *argv);
+	      ++argv;
+	      --argc;
+	    }
+	  else
+	    print_direc (direc_start, direc_length, field_width,
+			 precision, "");
+	  break;
+
+	case '\\':
+	  f += print_esc (f);
+	  break;
+
+	default:
+	  putchar (*f);
+	}
     }
+
+  return save_argc - argc;
+}
+
+int
+main (int argc, char **argv)
+{
+  char *format;
+  int args_used;
+
+  program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
+  exit_status = 0;
+
+  /* Don't recognize --help or --version if POSIXLY_CORRECT is set.  */
+  posixly_correct = (getenv ("POSIXLY_CORRECT") != NULL);
+  if (!posixly_correct)
+    parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+			AUTHORS, usage);
+
+  if (argc == 1)
+    {
+      fprintf (stderr, _("Usage: %s format [argument...]\n"), program_name);
+      exit (1);
+    }
+
+  format = argv[1];
+  argc -= 2;
+  argv += 2;
+
+  do
+    {
+      args_used = print_formatted (format, argc, argv);
+      argc -= args_used;
+      argv += args_used;
+    }
+  while (args_used > 0 && argc > 0);
+
+  if (argc > 0)
+    error (0, 0, _("warning: excess arguments have been ignored"));
+
+  exit (exit_status);
 }
