@@ -1,5 +1,5 @@
 /* whoami -- print effective userid
-   Copyright (C) 89, 90, 91, 92, 93, 1994 Free Software Foundation, Inc.
+   Copyright (C) 89,90, 1991-1997, 1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,80 +12,70 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Equivalent to `id -un'. */
 /* Written by Richard Mlynarik. */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <sys/types.h>
 #include <pwd.h>
 #include <getopt.h>
 
-#include "version.h"
 #include "system.h"
+#include "long-options.h"
+
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "whoami"
+
+#define AUTHORS "Richard Mlynarik"
 
 /* The name this program was run with. */
 char *program_name;
 
-/* If non-zero, display usage information and exit.  */
-static int show_help;
-
-/* If non-zero, print the version on standard output and exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
-  {"help", no_argument, &show_help, 1},
-  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
-static void
-usage (status)
-     int status;
+void
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("Usage: %s [OPTION]...\n", program_name);
-      printf ("\
+      printf (_("Usage: %s [OPTION]...\n"), program_name);
+      printf (_("\
+Print the user name associated with the current effective user id.\n\
+Same as id -un.\n\
 \n\
   --help      display this help and exit\n\
-  --version   output version information and exit\n\
-\n\
-Same as id -un.\n\
-");
+  --version   output version information and exit\n"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
 
-void
-main (argc, argv)
-     int argc;
-     char *argv[];
+int
+main (int argc, char **argv)
 {
   register struct passwd *pw;
   register uid_t uid;
   int c;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
-  while ((c = getopt_long (argc, argv, "", long_options, (int *) 0)) != EOF)
+  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
+
+  while ((c = getopt_long (argc, argv, "", long_options, NULL)) != -1)
     {
       switch (c)
 	{
@@ -97,16 +87,6 @@ main (argc, argv)
 	}
     }
 
-  if (show_version)
-    {
-      printf ("%s\n", version_string);
-      exit (0);
-    }
-
-  if (show_help)
-    usage (0);
-
-
   if (optind != argc)
     usage (1);
 
@@ -117,7 +97,7 @@ main (argc, argv)
       puts (pw->pw_name);
       exit (0);
     }
-  fprintf (stderr, "%s: cannot find username for UID %u\n",
+  fprintf (stderr, _("%s: cannot find username for UID %u\n"),
 	   program_name, (unsigned) uid);
   exit (1);
 }
